@@ -11,6 +11,7 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
+#include <fcntl.h>
 
 #ifndef _FILE_OFFSET_BITS
 #define _FILE_OFFSET_BITS 64
@@ -18,8 +19,8 @@
 
 class BufferedFile {
 public:
-	BufferedFile() { init(1024); }
-	BufferedFile(unsigned bufsize) { init(1024); }
+	BufferedFile() { init(2097152); }  // 2MB default buffer for genomic data
+	BufferedFile(unsigned bufsize) { init(bufsize); }
 	~BufferedFile();
 
 	// returns 0 on success, -1 on failure.
@@ -134,48 +135,76 @@ inline int BufferedFile::getc()
 
 inline uint64_t BufferedFile::read(void *ptr, uint64_t size)
 {
-	// is the new read already cached?
-	if (m_virt_pos >= m_sbuf_pos && m_virt_pos + (long)size <= m_ebuf_pos) {
-		memcpy(ptr, m_buf + m_virt_pos - m_sbuf_pos, size);
-		m_virt_pos += size;
-		return size;
-	}
+    // is the new read already cached?
+    if (m_virt_pos >= m_sbuf_pos && m_virt_pos + (long)size <= m_ebuf_pos) {
+        memcpy(ptr, m_buf + m_virt_pos - m_sbuf_pos, size);
+        m_virt_pos += size;
+        return size;
+    }
 
-	// we must perform a new read
-	if (m_phys_pos != m_virt_pos)
-		fseeko(m_fp, m_virt_pos, SEEK_SET);
+    // we must perform a new read
+    if (m_phys_pos != m_virt_pos)
+        fseeko(m_fp, m_virt_pos, SEEK_SET);
 
-	// We could check whether the buffer overlaps the reading section,
-	// but that would make the code slower and more complicated and would not
-	// give any real benefits considering that the buffer size is relatively small.
-	// In some cases (when the buffer overlaps the central part of the reading section
-	// that would be even harmful as it would result in two freads instead of one.
+    // For very large requests (>= buffer size), bypass cache and read directly
+    if (size >= m_bufsize) {
+#ifdef POSIX_FADV_SEQUENTIAL
+        int fd = fileno(m_fp);
+        posix_fadvise(fd, m_virt_pos, (off_t)size, POSIX_FADV_SEQUENTIAL);
+#endif
+        uint64_t bytes_read = fread(ptr, 1, size, m_fp);
+        m_virt_pos += bytes_read;
+        m_phys_pos = m_virt_pos;
+        // Invalidate buffer window after large direct read
+        m_sbuf_pos = m_ebuf_pos = 0;
+        if (!bytes_read && feof(m_fp))
+            m_eof = true;
+        return bytes_read;
+    }
 
-	// if size is smaller than bufsize => cache the read
-	if (size <= m_bufsize) {
-		uint64_t bytes_read = fread(m_buf, 1, m_bufsize, m_fp);
+    // If request is smaller than buffer size, perform cached read with modest prefetch
+    if (size < m_bufsize) {
+        uint64_t to_read = size;
+        const uint64_t modest_prefetch_small = 128 * 1024;  // small reads benefit
+        if (size < 100 * 1024 && to_read < modest_prefetch_small)
+            to_read = modest_prefetch_small;
 
-		m_phys_pos = m_virt_pos + bytes_read;
-		m_sbuf_pos = m_virt_pos;
-		m_ebuf_pos = m_phys_pos;
+        if (to_read > m_bufsize)
+            to_read = m_bufsize;
 
-		if (bytes_read > size)
-			bytes_read = size;
+        // Advise kernel about access pattern
+#ifdef POSIX_FADV_SEQUENTIAL
+        int fd = fileno(m_fp);
+        if (to_read >= 256 * 1024)
+            posix_fadvise(fd, m_virt_pos, (off_t)to_read, POSIX_FADV_SEQUENTIAL);
+        else
+            posix_fadvise(fd, m_virt_pos, (off_t)to_read, POSIX_FADV_RANDOM);
+#endif
 
-		m_virt_pos += bytes_read;
-		memcpy(ptr, m_buf, bytes_read);
-		if (!bytes_read && feof(m_fp))
-			m_eof = true;
-		return bytes_read;
-	}
+        uint64_t bytes_read = fread(m_buf, 1, to_read, m_fp);
 
-	// make the read without caching
-	uint64_t bytes_read = fread(ptr, 1, size, m_fp);
-	m_virt_pos += bytes_read;
-	m_phys_pos = m_virt_pos;
-	if (!bytes_read && feof(m_fp))
-		m_eof = true;
-	return bytes_read;
+        m_phys_pos = m_virt_pos + bytes_read;
+        m_sbuf_pos = m_virt_pos;
+        m_ebuf_pos = m_phys_pos;
+
+        uint64_t ret = bytes_read;
+        if (ret > size)
+            ret = size;
+
+        m_virt_pos += ret;
+        memcpy(ptr, m_buf, ret);
+        if (!bytes_read && feof(m_fp))
+            m_eof = true;
+        return ret;
+    }
+
+    // Fallback (should not be reached often)
+    uint64_t bytes_read = fread(ptr, 1, size, m_fp);
+    m_virt_pos += bytes_read;
+    m_phys_pos = m_virt_pos;
+    if (!bytes_read && feof(m_fp))
+        m_eof = true;
+    return bytes_read;
 }
 
 inline uint64_t BufferedFile::write(const void *ptr, uint64_t size)
