@@ -4,6 +4,7 @@
 #include <sys/types.h>
 
 #include "GenomeTrackArrays.h"
+#include "TrackIndex.h"
 
 const char *GenomeTrackArrays::SLICE_FUNCTION_NAMES[GenomeTrackArrays::NUM_S_FUNCS] = { "avg", "min", "max", "stddev", "sum", "quantile" };
 const int GenomeTrackArrays::RECORD_SIZE = 2 * sizeof(int64_t) + sizeof(long);
@@ -29,13 +30,63 @@ void GenomeTrackArrays::set_master_obj(GenomeTrackArrays *master_obj)
 void GenomeTrackArrays::init_read(const char *filename, int chromid)
 {
 	finish_writing();
-	m_bfile.close();
+	m_base_offset = 0;
 	m_loaded = false;
 	m_is_writing = false;
 	m_intervals.clear();
 	m_vals_pos.clear();
-	if (!m_master_obj) 
-		read_type(filename);
+
+	if (!m_master_obj) {
+		// Check for indexed format FIRST
+		const std::string track_dir = GenomeTrack::get_track_dir(filename);
+		const std::string idx_path = track_dir + "/track.idx";
+
+		struct stat idx_st;
+		if (stat(idx_path.c_str(), &idx_st) == 0) {
+			// --- INDEXED PATH ---
+			const std::string dat_path  = track_dir + "/track.dat";
+
+			// Reopen file if path or mode changed
+			if (!m_dat_open || m_dat_path != dat_path || m_dat_mode != "rb") {
+				m_bfile.close();
+				if (m_bfile.open(dat_path.c_str(), "rb"))
+					TGLError<GenomeTrackArrays>("Cannot open %s: %s", dat_path.c_str(), strerror(errno));
+				m_dat_open = true;
+				m_dat_path = dat_path;
+				m_dat_mode = "rb";
+			}
+
+			auto idx   = get_track_index(track_dir);
+			if (!idx)
+				TGLError<GenomeTrackArrays>("Failed to load track index for %s", track_dir.c_str());
+
+			auto entry = idx->get_entry(chromid);
+			if (!entry || entry->length == 0) {
+				// Chromosome not in index or empty contig - treat as empty
+				m_chromid = chromid;
+				return;
+			}
+
+			if (m_bfile.seek(entry->offset, SEEK_SET))
+				TGLError<GenomeTrackArrays>("Failed to seek to offset %llu in %s",
+					(unsigned long long)entry->offset, dat_path.c_str());
+
+			// Read the format signature (similar to read_type)
+			int32_t signature = 0;
+			if (m_bfile.read(&signature, sizeof(signature)) != sizeof(signature))
+				TGLError<GenomeTrackArrays>("Failed to read arrays track header in %s", dat_path.c_str());
+			if (signature != GenomeTrack::FORMAT_SIGNATURES[ARRAYS])
+				TGLError<GenomeTrackArrays>("Invalid arrays track header in %s", dat_path.c_str());
+
+			m_base_offset = entry->offset; 
+		} else {
+			// --- PER-CHROMOSOME PATH ---
+			m_bfile.close();
+			m_dat_open = false;
+			read_type(filename);
+		}
+	}
+
 	m_chromid = chromid;
 }
 
@@ -108,6 +159,11 @@ void GenomeTrackArrays::read_intervals_map()
 			TGLError<GenomeTrackArrays>("Invalid format of %s track file %s", TYPE_NAMES[ARRAYS], m_bfile.file_name().c_str());
 		}
 
+		// Adjust m_intervals_pos by base_offset for indexed format
+		if (m_base_offset > 0) {
+			m_intervals_pos += m_base_offset;
+		}
+
 		// read number of intervals
 		if (m_bfile.seek(m_intervals_pos, SEEK_SET))
 			TGLError<GenomeTrackArrays>("Failed to read %s track file %s: %s", TYPE_NAMES[ARRAYS], m_bfile.file_name().c_str(), strerror(errno));
@@ -135,6 +191,11 @@ void GenomeTrackArrays::read_intervals_map()
 				TGLError<GenomeTrackArrays>("Invalid format of %s track file %s", TYPE_NAMES[ARRAYS], m_bfile.file_name().c_str());
 			}
 
+			// Adjust m_vals_pos by base_offset for indexed format
+			if (m_base_offset > 0) {
+				m_vals_pos[i] += m_base_offset;
+			}
+
 			interval.chromid = m_chromid;
 
 			if (interval.start < 0 || interval.start >= interval.end || (i && interval.start < m_intervals[i - 1].end) ||
@@ -154,6 +215,10 @@ void GenomeTrackArrays::read_interval(const GInterval &interval)
 
 	for (vector<GenomeTrackArrays *>::iterator itrack = m_dependent_objs.begin(); itrack != m_dependent_objs.end(); ++itrack) {
 		(*itrack)->m_last_avg = (*itrack)->m_last_nearest = (*itrack)->m_last_min = (*itrack)->m_last_max = (*itrack)->m_last_stddev = (*itrack)->m_last_sum = numeric_limits<float>::quiet_NaN();
+		if ((*itrack)->m_functions[MAX_POS])
+			(*itrack)->m_last_max_pos = numeric_limits<double>::quiet_NaN();
+		if ((*itrack)->m_functions[MIN_POS])
+			(*itrack)->m_last_min_pos = numeric_limits<double>::quiet_NaN();
 
 		if ((*itrack)->m_use_quantile)
 			(*itrack)->m_sp.reset();
@@ -222,6 +287,16 @@ void GenomeTrackArrays::read_interval(const GInterval &interval)
 	}
 }
 
+double GenomeTrackArrays::last_max_pos() const
+{
+	return m_last_max_pos;
+}
+
+double GenomeTrackArrays::last_min_pos() const
+{
+	return m_last_min_pos;
+}
+
 void GenomeTrackArrays::write_next_interval(const GInterval &interval, const ArrayVals::const_iterator &iarray_vals_begin, const ArrayVals::const_iterator &iarray_vals_end)
 {
 	if (iarray_vals_begin == iarray_vals_end) 
