@@ -8,8 +8,11 @@
 #ifndef STATQUADTREECACHED_H_
 #define STATQUADTREECACHED_H_
 
-#include <ext/hash_map>
+#include <cstdint>
+#include <inttypes.h>
+
 #include <list>
+#include <unordered_map>
 #include <vector>
 
 #include "DiagonalBand.h"
@@ -34,7 +37,7 @@
 // 5. Unlike StatQuadTree that must be entirely stored in the memory, StatQuadTreeCached may load some parts of the quad tree
 //    on demand in order to answer "get_stat" or "do_intersect" queries. Each part that StatQuadTreeCached loads is called a "chunk".
 //    Chunks are stored in the memory and can be loaded or unloaded in accordance to their recent usage.
-//    Chunk size and the maximal number of chunks StatQuadTreeCached stores at any given moment in the memory is controled by init() function.
+//    Chunk size and the maximal number of chunks StatQuadTreeCached stores at any given moment in the memory is controlled by init() function.
 // 6. Even on small trees StatQuadTreeCached may be more efficient than StatQuadTree since it stores objects in the memory in a more compact and
 //    efficient way in the memo. This efficiency has a price: as mentioned earlier new objects cannot be inserted to StatQuadTreeCached.
 // 7. StatQuadTreeCached replaces StatQuadTree::get_objs() function by an Iterator mechanism. The need to store in the memory all the objects
@@ -53,7 +56,7 @@ public:
 	typedef typename StatQuadTree<T, Size>::Stat Stat;
 
 	// chunk size 0 means there's no caching at all
-	StatQuadTreeCached() : m_chunk_size(0), m_max_num_chunks(0), m_num_chunks(0), m_bfile(NULL), m_uptr(NULL), m_local2global_id(NULL) {}
+	StatQuadTreeCached() : m_chunk_size(0), m_max_num_chunks(0), m_num_chunks(0), m_bfile(NULL), m_local2global_id(NULL), m_uptr(NULL) {}
 	StatQuadTreeCached(int64_t chunk_size, int64_t max_num_chunks) : m_uptr(NULL) { init(chunk_size, max_num_chunks); }
 
 	~StatQuadTreeCached() { clear(); }
@@ -170,9 +173,9 @@ private:
 		int64_t   m_quad_ptr;
 	};
 
-	typedef list<Chunk>                                  Chunks;
-	typedef hash_map<int64_t, typename Chunks::iterator> Fpos2ichunk;
-	typedef vector<int64_t>                              Stacked_chunks_fpos;
+	typedef list<Chunk>                                       Chunks;
+	typedef unordered_map<int64_t, typename Chunks::iterator> Fpos2ichunk;
+	typedef vector<int64_t>                                   Stacked_chunks_fpos;
 
 	int64_t             m_chunk_size;
 	int64_t             m_max_num_chunks;
@@ -538,7 +541,7 @@ int64_t StatQuadTreeCached<T, Size>::analyze_n_serialize_subtree(BufferedFile &f
 		size += sizeof(Node);
 	}
 
-	if (m_chunk_size && size > m_chunk_size || &node == &qtree.m_nodes.front()) {
+	if ((m_chunk_size && size > m_chunk_size) || &node == &qtree.m_nodes.front()) {
 		int64_t chunk_start_fpos = file.tell();
 		int64_t top_node_offset = 0;
 
@@ -566,10 +569,8 @@ int64_t StatQuadTreeCached<T, Size>::serialize_subtree(BufferedFile &file, const
 	int64_t top_node_fpos;
 
 	if (node.is_leaf) {
-		Leaf new_leaf;
+		Leaf new_leaf{};
 
-		// ensure that stucture's members are padded with zeroes (and not some junk) otherwise there will be no binary consistency between the files with identical quad trees
-		memset(&new_leaf, 0, sizeof(new_leaf));
 		new_leaf.is_leaf = true;
 		new_leaf.stat = node.stat;
 		new_leaf.arena = node.arena;
@@ -589,10 +590,8 @@ int64_t StatQuadTreeCached<T, Size>::serialize_subtree(BufferedFile &file, const
 			file.write(&obj, sizeof(obj));
 		}
 	} else {
-		Node new_node;
+		Node new_node{};
 
-		// ensure that stucture's members are padded with zeroes (and not some junk) otherwise there will be no binary consistency between the files with identical quad trees
-		memset(&new_node, 0, sizeof(new_node));
 		new_node.is_leaf = false;
 		new_node.stat = node.stat;
 		new_node.arena = node.arena;
@@ -902,39 +901,39 @@ void StatQuadTreeCached<T, Size>::debug_print_tree()
 {
 	if (!empty())
 		debug_print_tree(m_root_chunk, m_root_chunk.top_node, 0);
-	printf("Objs: %ld\n", m_num_objs);
+	fprintf(stderr, "Objs: %llu\n", m_num_objs);
 }
 
 template <class T, class Size>
 void StatQuadTreeCached<T, Size>::debug_print_tree(const Chunk &chunk, NodeBase *node_base, unsigned depth)
 {
 	Rectangle arena;
-	printf("\n%*sArena: %s\n", depth * 2, "", node_base->arena.debug_str());
-	printf("%*sIs leaf?: %d\n", (depth + 1) * 2, "", node_base->is_leaf);
-	printf("%*sArea occupied: %ld\n", (depth + 1) * 2, "", node_base->stat.occupied_area);
-	printf("%*sAvg: %g\tMin: %g\tMax: %g\n", (depth + 1) * 2, "", node_base->stat.occupied_area / (double)node_base->stat.weighted_sum, node_base->stat.min_val, node_base->stat.max_val);
+	fprintf(stderr, "\n%*sArena: %s\n", depth * 2, "", node_base->arena.debug_str());
+	fprintf(stderr, "%*sIs leaf?: %d\n", (depth + 1) * 2, "", node_base->is_leaf);
+	fprintf(stderr, "%*sArea occupied: %" PRId64 "\n", (depth + 1) * 2, "", node_base->stat.occupied_area);
+	fprintf(stderr, "%*sAvg: %g\tMin: %g\tMax: %g\n", (depth + 1) * 2, "", node_base->stat.occupied_area / (double)node_base->stat.weighted_sum, node_base->stat.min_val, node_base->stat.max_val);
 
 	if (node_base->is_leaf) {
 		Leaf *leaf = (Leaf *)node_base;
 		Obj *objs = get_objs(leaf);
 
-		printf("%*sKids: %d\n", (depth + 1) * 2, "", leaf->num_objs);
+		fprintf(stderr, "%*sKids: %d\n", (depth + 1) * 2, "", leaf->num_objs);
 		for (unsigned i = 0; i < leaf->num_objs; ++i) {
-			printf("%*s%s", (depth + 2) * 2, "", objs[i].obj.debug_str());
-			printf("\n");
+			fprintf(stderr, "%*s%s", (depth + 2) * 2, "", objs[i].obj.debug_str());
+			fprintf(stderr, "\n");
 		}
 	} else {
 		Node *node = (Node *)node_base;
 		for (int iquad = 0; iquad < NUM_QUADS; ++iquad) {
 			QuadRetriever qr(this, chunk, node->kid_ptr[iquad]);
 			if (iquad == NW)
-				printf("%*sNW node\n", (depth + 1) * 2, "");
+				fprintf(stderr, "%*sNW node\n", (depth + 1) * 2, "");
 			else if (iquad == NE)
-				printf("%*sNE node\n", (depth + 1) * 2, "");
+				fprintf(stderr, "%*sNE node\n", (depth + 1) * 2, "");
 			else if (iquad == SW)
-				printf("%*sSW node\n", (depth + 1) * 2, "");
+				fprintf(stderr, "%*sSW node\n", (depth + 1) * 2, "");
 			else
-				printf("%*sSE node\n", (depth + 1) * 2, "");
+				fprintf(stderr, "%*sSE node\n", (depth + 1) * 2, "");
 			debug_print_tree(qr.chunk(), qr.quad(), depth + 1);
 		}
 	}
