@@ -20,17 +20,39 @@
 #include <R.h>
 #include <Rinternals.h>
 
+// Undefine R macros that conflict with C++ standard library
+#ifdef length
+#undef length
+#endif
+#ifdef error
+#undef error
+#endif
+#ifdef warning
+#undef warning
+#endif
+
 #include "BinFinder.h"
+#include "Filter.h"
+#include "FilterRegistry.h"
 #include "GenomeTrack.h"
 #include "GenomeTrackArrays.h"
+#include "GenomeTrackInMemory.h"
 #include "GInterval.h"
 #include "GInterval2D.h"
 #include "TrackExpressionIteratorBase.h"
 #include "PWMScorer.h"
 #include "KmerCounter.h"
+#include "MaskedBpCounter.h"
+#include "GenomeSeqFetch.h"
 #include "rdbinterval.h"
 #include "rdbutils.h"
 
+// Forward declarations to avoid circular dependency
+class TrackVarProcessor;
+class IntervVarProcessor;
+class ValueVarProcessor;
+class SequenceVarProcessor;
+
 using namespace std;
 
 class TrackExpressionVars {
@@ -105,8 +127,26 @@ public:
             PWM,
             PWM_MAX,
             PWM_MAX_POS,
+            PWM_COUNT,
             KMER_COUNT,
-            KMER_FRAC, 
+            KMER_FRAC,
+            MASKED_COUNT,
+            MASKED_FRAC,
+            MAX_POS_ABS,
+            MAX_POS_REL,
+            MIN_POS_ABS,
+            MIN_POS_REL,
+            EXISTS,
+            SIZE,
+            SAMPLE,
+            SAMPLE_POS_ABS,
+            SAMPLE_POS_REL,
+            FIRST,
+            FIRST_POS_ABS,
+            FIRST_POS_REL,
+            LAST,
+            LAST_POS_ABS,
+            LAST_POS_REL,
             NUM_FUNCS
         };
 
@@ -122,16 +162,19 @@ public:
         Track_n_imdf       *track_n_imdf;
         std::unique_ptr<PWMScorer> pwm_scorer;
         std::unique_ptr<KmerCounter> kmer_counter;
+        std::unique_ptr<MaskedBpCounter> masked_counter;
         char strand;
         // When the virtual track is sequence-based (PWM/KMER) we may still
         // have per-vtrack iterator modifiers; store them here.
         Iterator_modifier1D *seq_imdf1d{NULL};
+        // Filter for masking genomic regions (applied after iterator modifiers)
+        std::shared_ptr<Genome1DFilter> filter;
     };
 
     typedef vector<Track_var> Track_vars;
 
     struct Interv_var {
-        enum Val_func { DIST, DIST_CENTER, COVERAGE, NUM_FUNCS };
+        enum Val_func { DIST, DIST_CENTER, DIST_EDGE, COVERAGE, NEIGHBOR_COUNT, NUM_FUNCS };
 
         static const char *FUNC_NAMES[NUM_FUNCS];
 
@@ -146,15 +189,50 @@ public:
         GIntervals::const_iterator siinterv;
         GIntervals::const_iterator eiinterv;
         double                     dist_margin;
+        // Filter for masking genomic regions (applied after iterator modifiers)
+        std::shared_ptr<Genome1DFilter> filter;
     };
 
     typedef vector<Interv_var> Interv_vars;
 
+    struct Value_var {
+        enum Val_func {
+            AVG, MIN, MAX, STDDEV, SUM, QUANTILE,
+            NEAREST,
+            EXISTS, SIZE,
+            FIRST, LAST, SAMPLE,
+            FIRST_POS_ABS, FIRST_POS_REL,
+            LAST_POS_ABS, LAST_POS_REL,
+            SAMPLE_POS_ABS, SAMPLE_POS_REL,
+            MIN_POS_ABS, MIN_POS_REL,
+            MAX_POS_ABS, MAX_POS_REL,
+            NUM_FUNCS
+        };
+
+        static const char *FUNC_NAMES[NUM_FUNCS];
+
+        string                     var_name;
+        SEXP                       rvar{R_NilValue};
+        double                    *var;
+        Iterator_modifier1D       *imdf1d;
+        Val_func                   val_func;
+        double                     percentile;
+
+        // Track object that holds the intervals and values
+        std::shared_ptr<GenomeTrackInMemory> track;
+
+        // Filter for masking genomic regions (applied after iterator modifiers)
+        std::shared_ptr<Genome1DFilter> filter;
+    };
+
+    typedef vector<Value_var> Value_vars;
+
 	TrackExpressionVars(rdb::IntervUtils &iu);
 	~TrackExpressionVars();
 
 	unsigned get_num_track_vars() const { return m_track_vars.size(); }
 	unsigned get_num_interv_vars() const { return m_interv_vars.size(); }
+	unsigned get_num_value_vars() const { return m_value_vars.size(); }
 
 	const string &get_track_name(unsigned ivar) const { return m_track_vars[ivar].track_n_imdf->name; }
 	GenomeTrack::Type get_track_type(unsigned ivar) const { return m_track_vars[ivar].track_n_imdf->type; }
@@ -166,6 +244,12 @@ public:
 
     bool is_seq_variable(unsigned ivar) const;
 
+    // Helper methods to check variable function types
+    static bool is_sequence_based_function(Track_var::Val_func func);
+    static bool is_pwm_function(Track_var::Val_func func);
+    static bool is_kmer_function(Track_var::Val_func func);
+    static bool is_masked_function(Track_var::Val_func func);
+
 	void set_vars(const GInterval &interval, unsigned idx);
 	void set_vars(const GInterval2D &interval, const DiagonalBand &band, unsigned idx);
 
@@ -174,6 +258,7 @@ private:
     string                  m_groot;
 	Track_vars              m_track_vars;
 	Interv_vars             m_interv_vars;
+	Value_vars              m_value_vars;
 	Track_n_imdfs           m_track_n_imdfs;
 	Iterator_modifiers1D    m_imdfs1d;
 	Iterator_modifiers2D    m_imdfs2d;
@@ -181,6 +266,15 @@ private:
 	GInterval2D             m_interval2d;
 	DiagonalBand            m_band;
 
+	// Shared sequence fetcher for all sequence-based vtracks to enable caching
+	GenomeSeqFetch          m_shared_seqfetch;
+
+	// Processors for different variable types (using pointers to avoid circular dependency)
+	std::unique_ptr<TrackVarProcessor>       m_track_processor;
+	std::unique_ptr<IntervVarProcessor>      m_interv_processor;
+	std::unique_ptr<ValueVarProcessor>       m_value_processor;
+	std::unique_ptr<SequenceVarProcessor>    m_sequence_processor;
+
 	void                 parse_imdf(SEXP rvtrack, const string &vtrack, Iterator_modifier1D *imdf1d, Iterator_modifier2D *imdf2d);
 	Iterator_modifier1D *add_imdf(const Iterator_modifier1D &imdf1d);
 	Iterator_modifier2D *add_imdf(const Iterator_modifier2D &imdf2d);
@@ -191,12 +285,17 @@ private:
 	void                 add_vtrack_var(const string &track, SEXP rvtrack);
 	Track_var           &add_vtrack_var_src_track(SEXP rvtrack, const string &vtrack, const string &track);
 	Interv_var          &add_vtrack_var_src_interv(SEXP rvtrack, const string &vtrack, GIntervals &intervs1d, GIntervals2D &intervs2d);
+	Value_var           &add_vtrack_var_src_value(SEXP rvtrack, const string &vtrack, GIntervals &intervs, vector<float> &vals);
+	void                 attach_filter_to_var(SEXP rvtrack, const string &vtrack, Track_var &var);
+	void                 attach_filter_to_var(SEXP rvtrack, const string &vtrack, Interv_var &var);
+	void                 attach_filter_to_var(SEXP rvtrack, const string &vtrack, Value_var &var);
 	void                 register_track_functions();
 
 	void start_chrom(const GInterval &interval);
 	void start_chrom(const GInterval2D &interval);
 	void set_vars(unsigned idx);
 
+
 	bool is_var(const string &str, uint64_t start, uint64_t end) const { return (!start || !rdb::is_R_var_char(str[start - 1])) && (end == str.size() || !rdb::is_R_var_char(str[end])); }
 
     static int findListElementIndex(SEXP list, const char* name) {
@@ -265,8 +364,32 @@ inline bool TrackExpressionVars::is_seq_variable(unsigned ivar) const {
     return m_track_vars[ivar].val_func == Track_var::PWM ||
            m_track_vars[ivar].val_func == Track_var::PWM_MAX ||
            m_track_vars[ivar].val_func == Track_var::PWM_MAX_POS ||
+           m_track_vars[ivar].val_func == Track_var::PWM_COUNT ||
            m_track_vars[ivar].val_func == Track_var::KMER_COUNT ||
-           m_track_vars[ivar].val_func == Track_var::KMER_FRAC;
+           m_track_vars[ivar].val_func == Track_var::KMER_FRAC ||
+           m_track_vars[ivar].val_func == Track_var::MASKED_COUNT ||
+           m_track_vars[ivar].val_func == Track_var::MASKED_FRAC;
+}
+
+// Helper methods to check variable function types
+inline bool TrackExpressionVars::is_sequence_based_function(Track_var::Val_func func) {
+    return func == Track_var::PWM || func == Track_var::PWM_MAX ||
+           func == Track_var::PWM_MAX_POS || func == Track_var::PWM_COUNT ||
+           func == Track_var::KMER_COUNT || func == Track_var::KMER_FRAC ||
+           func == Track_var::MASKED_COUNT || func == Track_var::MASKED_FRAC;
+}
+
+inline bool TrackExpressionVars::is_pwm_function(Track_var::Val_func func) {
+    return func == Track_var::PWM || func == Track_var::PWM_MAX ||
+           func == Track_var::PWM_MAX_POS || func == Track_var::PWM_COUNT;
+}
+
+inline bool TrackExpressionVars::is_kmer_function(Track_var::Val_func func) {
+    return func == Track_var::KMER_COUNT || func == Track_var::KMER_FRAC;
+}
+
+inline bool TrackExpressionVars::is_masked_function(Track_var::Val_func func) {
+    return func == Track_var::MASKED_COUNT || func == Track_var::MASKED_FRAC;
 }
 
 #endif /* TRACKEXPRESSIONVARS_H_ */
